package com.company.exceluploadsample.view.order;

import com.company.exceluploadsample.entity.Customer;
import com.company.exceluploadsample.entity.Order;

import com.company.exceluploadsample.view.main.MainView;

import com.vaadin.flow.router.Route;
import io.jmix.dataimport.DataImporter;
import io.jmix.dataimport.InputDataFormat;
import io.jmix.dataimport.configuration.DuplicateEntityPolicy;
import io.jmix.dataimport.configuration.ImportConfiguration;
import io.jmix.dataimport.configuration.ImportTransactionStrategy;
import io.jmix.dataimport.configuration.mapping.ReferenceImportPolicy;
import io.jmix.dataimport.result.ImportResult;
import io.jmix.flowui.component.upload.FileStorageUploadField;
import io.jmix.flowui.component.upload.receiver.FileTemporaryStorageBuffer;
import io.jmix.flowui.kit.component.upload.event.FileUploadSucceededEvent;
import io.jmix.flowui.upload.TemporaryStorage;
import io.jmix.flowui.view.*;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.File;
import java.io.FileInputStream;
import java.nio.file.Files;
import java.io.IOException;
import java.nio.file.Path;
import java.util.UUID;

@Route(value = "orders", layout = MainView.class)
@ViewController("xls_Order.list")
@ViewDescriptor("order-list-view.xml")
@LookupComponent("ordersDataGrid")
@DialogMode(width = "64em")
public class OrderListView extends StandardListView<Order> {
    private static final Logger log = LoggerFactory.getLogger(OrderListView.class);
    @Autowired
    private TemporaryStorage temporaryStorage;

    @Autowired
    private DataImporter dataImporter;

    @Subscribe("fileUploadBtn")
    public void onFileUploadBtnFileUploadSucceeded(final FileUploadSucceededEvent<FileStorageUploadField> event) {

        if (event.getReceiver() instanceof FileTemporaryStorageBuffer buffer) {
            UUID fileId = buffer.getFileData().getFileInfo().getId();
            log.info("FileId: " + fileId);

            File file = temporaryStorage.getFile(fileId);
            log.info("File from temp storage: " + file);

            ImportConfiguration importConfiguration = ImportConfiguration.builder(Customer.class, InputDataFormat.XLSX)
                    .addSimplePropertyMapping("orderNo", "Order No")
                    .addSimplePropertyMapping("orderDate", "Order Date")
                    .addSimplePropertyMapping("amount", "Order Amount")
                    .addReferencePropertyMapping("customer", "customer Code", "customerCode", ReferenceImportPolicy.IGNORE_IF_MISSING)
                    .addUniqueEntityConfiguration(DuplicateEntityPolicy.UPDATE, "orderNo", "orderDate", "customer.customerCode")
                    .withDateFormat("yyyy/MM/dd")
                    .withTransactionStrategy(ImportTransactionStrategy.TRANSACTION_PER_ENTITY)
                    .build();

            try (FileInputStream fileInputStream = new FileInputStream(file)) {
                byte[] byteArray = new byte[(int) file.length()];
                fileInputStream.read(byteArray);
                // Now, byteArray contains the file data in bytes
                ImportResult importResult = dataImporter.importData(importConfiguration, byteArray);

                temporaryStorage.deleteFile(fileId);
                log.info("File is deleted from temp storage: " + file);
            } catch (IOException e) {
                e.printStackTrace();
            }



        }



    }


}